@@ -1,15 +1,15 @@
 #!/usr/bin/env python3
 
-__all__ = ['to_tensor', 'to_numpy', 'to_list']
-
-from trojanzoo import __version__
-
 from trojanvision import environ as environ
 from trojanvision import datasets as datasets
 from trojanvision import models as models
 from trojanvision import trainer as trainer
+
 from trojanvision import attacks as attacks
 from trojanvision import defenses as defenses
 from trojanvision import marks as marks
 
-from trojanzoo.utils import to_tensor, to_numpy, to_list
+from trojanzoo import __version__
+from trojanzoo import summary
+
+__all__ = ['summary']
