@@ -13,7 +13,6 @@ import numpy as np
 import os
 import PIL.Image as Image
 
-from typing import Optional, Union
 import argparse
 from trojanzoo.configs import Config
 from collections.abc import Callable
@@ -23,31 +22,33 @@ dir_path = os.path.dirname(__file__)
 
 def get_edge_color(
     mark: torch.Tensor,
-    mark_background_color: Union[str, torch.Tensor] = 'auto'
-) -> Optional[torch.Tensor]:
+    mark_background_color: str | torch.Tensor = 'auto'
+) -> torch.Tensor | None:
     # if any pixel is not fully opaque
     if not mark[-1].allclose(torch.ones_like(mark[-1]), atol=1e-3):
         return None
     mark = mark[:-1]    # remove alpha channel
-    if isinstance(mark_background_color, torch.Tensor):    # TODO: python 3.10 match
-        return torch.as_tensor(mark_background_color).expand(mark.size(0))
-    elif mark_background_color == 'black':
-        return torch.zeros(mark.size(0))
-    elif mark_background_color == 'white':
-        return torch.ones(mark.size(0))
-    elif mark_background_color == 'auto':
-        if mark.flatten(1).std(1).max() < 1e-3:
-            return None
-        else:
-            _list = [mark[:, 0, :], mark[:, -1, :],
-                     mark[:, :, 0], mark[:, :, -1]]
-            return torch.cat(_list, dim=1).mode(dim=-1)[0]
-    raise ValueError(f'{mark_background_color=:s}')
+    match mark_background_color:
+        case torch.Tensor():
+            return torch.as_tensor(mark_background_color).expand(mark.size(0))
+        case 'black':
+            return torch.zeros(mark.size(0))
+        case 'white':
+            return torch.ones(mark.size(0))
+        case 'auto':
+            if mark.flatten(1).std(1).max() < 1e-3:
+                return None
+            else:
+                _list = [mark[:, 0, :], mark[:, -1, :],
+                         mark[:, :, 0], mark[:, :, -1]]
+                return torch.cat(_list, dim=1).mode(dim=-1)[0]
+        case _:
+            raise ValueError(f'{mark_background_color=:s}')
 
 
 def update_mark_alpha_channel(
     mark: torch.Tensor,
-    mark_background_color: Optional[torch.Tensor]
+    mark_background_color: torch.Tensor | None = None
 ) -> torch.Tensor:
     if mark_background_color is None:
         return mark
@@ -235,7 +236,7 @@ class Watermark(BasicObject):
         return group
 
     def __init__(self, mark_path: str = 'square_white.png',
-                 data_shape: list[int] = None, mark_background_color: Union[str, torch.Tensor] = 'auto',
+                 data_shape: list[int] = None, mark_background_color: str | torch.Tensor = 'auto',
                  mark_alpha: float = 1.0, mark_height: int = 3, mark_width: int = 3,
                  mark_height_offset: int = 0, mark_width_offset: int = 0,
                  mark_random_init: bool = False, mark_random_pos: bool = False,
@@ -355,8 +356,8 @@ class Watermark(BasicObject):
 
     def load_mark(
         self,
-        mark_img: Union[str, Image.Image, np.ndarray, torch.Tensor],
-        mark_background_color: Union[str, torch.Tensor, None] = 'auto',
+        mark_img: str | Image.Image | np.ndarray | torch.Tensor,
+        mark_background_color: None | str | torch.Tensor = 'auto',
         already_processed: bool = False
     ) -> torch.Tensor:
         r"""Load watermark tensor from image :attr:`mark_img`,
@@ -413,13 +414,41 @@ class Watermark(BasicObject):
 
 
 def add_argument(parser: argparse.ArgumentParser) -> argparse._ArgumentGroup:
+    r"""
+    | Add watermark arguments to argument parser.
+    | For specific arguments implementation, see :meth:`Watermark.add_argument()`.
+
+    Args:
+        parser (argparse.ArgumentParser): The parser to add arguments.
+    """
     group = parser.add_argument_group('{yellow}mark{reset}'.format(**ansi))
     return Watermark.add_argument(group)
 
 
 def create(mark_path: str = None, data_shape: list[int] = None,
-           dataset_name: str = None, dataset: Union[str, ImageSet] = None,
-           config: Config = config, **kwargs):
+           dataset_name: str = None, dataset: str | ImageSet = None,
+           config: Config = config, **kwargs) -> Watermark:
+    r"""
+    | Create a watermark instance.
+    | For arguments not included in :attr:`kwargs`,
+      use the default values in :attr:`config`.
+    | For watermark implementation, see :class:`Watermark`.
+
+    Args:
+        mark_path (str):
+            | Path to watermark image or npy file.
+              There are some preset marks in the package.
+            | Defaults to ``'square_white.png'``.
+        data_shape (list[int]): The shape of image data ``[C, H, W]``.
+        dataset_name (str): The dataset name.
+        dataset (str): The alias of `dataset_name`.
+        config (Config): The default parameter config.
+        **kwargs: Keyword arguments
+            passed to dataset init method.
+
+    Returns:
+        Watermark: Watermark instance.
+    """
     if data_shape is None:
         assert isinstance(dataset, ImageSet), 'Please specify data_shape or dataset'
         data_shape = dataset.data_shape
