@@ -17,7 +17,6 @@ from torchvision.transforms import Normalize
 import re
 
 from typing import TYPE_CHECKING
-from typing import Union
 from trojanzoo.utils.model import ExponentialMovingAverage
 from torch.optim.optimizer import Optimizer
 from torch.optim.lr_scheduler import _LRScheduler
@@ -53,24 +52,26 @@ def set_first_layer_channel(model: nn.Module,
         if len(list(module.children())):
             set_first_layer_channel(module, channel=channel)
             break
-        elif isinstance(module, nn.Conv2d):
-            if module.in_channels == channel:
-                break
-            keys = ['out_channels', 'kernel_size', 'bias', 'stride', 'padding']
-            args = {key: getattr(module, key) for key in keys}
-            args['device'] = module.weight.device
-            args.update(kwargs)
-            new_conv = nn.Conv2d(in_channels=channel, **args)
-            setattr(model, name, new_conv)
-        elif isinstance(module, nn.Linear):
-            if module.in_features == channel:
-                break
-            keys = ['out_features', 'bias']
-            args = {key: getattr(module, key) for key in keys}
-            args['device'] = module.weight.device
-            args.update(kwargs)
-            new_linear = nn.Linear(in_features=channel, **args)
-            setattr(model, name, new_linear)
+        else:
+            match module:
+                case nn.Conv2d():
+                    if module.in_channels == channel:
+                        break
+                    keys = ['out_channels', 'kernel_size', 'bias', 'stride', 'padding']
+                    args = {key: getattr(module, key) for key in keys}
+                    args['device'] = module.weight.device
+                    args.update(kwargs)
+                    new_conv = nn.Conv2d(in_channels=channel, **args)
+                    setattr(model, name, new_conv)
+                case nn.Linear():
+                    if module.in_features == channel:
+                        break
+                    keys = ['out_features', 'bias']
+                    args = {key: getattr(module, key) for key in keys}
+                    args['device'] = module.weight.device
+                    args.update(kwargs)
+                    new_linear = nn.Linear(in_features=channel, **args)
+                    setattr(model, name, new_linear)
         break
 
 
@@ -201,15 +202,16 @@ class ImageModel(Model):
                            help='sgm gamma (default: 1.0)')
         return group
 
-    def __init__(self, name: str = 'imagemodel', layer: Union[int, str] = '',
-                 model: Union[type[_ImageModel], _ImageModel] = _ImageModel,
+    def __init__(self, name: str = 'imagemodel', layer: int | str = '',
+                 model: type[_ImageModel] | _ImageModel = _ImageModel,
                  dataset: ImageSet = None, data_shape: list[int] = None,
                  adv_train: str = None, adv_train_random_init: bool = False, adv_train_eval_random_init: bool = None,
                  adv_train_iter: int = 7, adv_train_alpha: float = 2 / 255, adv_train_eps: float = 8 / 255,
                  adv_train_eval_iter: int = None, adv_train_eval_alpha: float = None, adv_train_eval_eps: float = None,
                  adv_train_trades_beta: float = 6.0,
                  norm_layer: str = 'bn', sgm: bool = False, sgm_gamma: float = 1.0,
-                 norm_par: dict[str, list[float]] = None, suffix: str = None, **kwargs):
+                 norm_par: dict[str, list[float]] = None, suffix: str = None,
+                 modify_first_layer_channel: bool = True, **kwargs):
         name = self.get_name(name, layer=layer)
         if norm_par is None and isinstance(dataset, ImageSet):
             norm_par = None if dataset.normalize else dataset.norm_par
@@ -227,8 +229,9 @@ class ImageModel(Model):
             assert isinstance(dataset, ImageSet), 'Please specify data_shape or dataset'
             data_shape = dataset.data_shape
         args = {'padding': 3} if 'vgg' in name else {}  # TODO: so ugly
-        set_first_layer_channel(self._model.features,
-                                channel=data_shape[0], **args)
+        if modify_first_layer_channel:
+            set_first_layer_channel(self._model.features,
+                                    channel=data_shape[0], **args)
 
         self.sgm: bool = sgm
         self.sgm_gamma: float = sgm_gamma
@@ -283,7 +286,7 @@ class ImageModel(Model):
                          reduction='batchmean')
 
     @classmethod
-    def get_name(cls, name: str, layer: Union[int, str] = '') -> str:
+    def get_name(cls, name: str, layer: int | str = '') -> str:
         r"""A useful function to combine :attr:`name` and :attr:`layer`.
 
         Note:
@@ -420,30 +423,33 @@ class ImageModel(Model):
             _label = [_label] * len(_input)
         _label = torch.as_tensor(_label, device=_input.device)
         heatmap = _input    # linting purpose
-        if method == 'grad_cam':    # TODO: python 3.10 match
-            feats = self._model.get_fm(_input).detach()   # (N, C', H', W')
-            feats.requires_grad_()
-            _output: torch.Tensor = self._model.pool(feats)   # (N, C', 1, 1)
-            _output = self._model.flatten(_output)   # (N, C')
-            _output = self._model.classifier(_output)   # (N, num_classes)
-            _output = _output.gather(dim=1, index=_label.unsqueeze(1)).sum()
-            grad = torch.autograd.grad(_output, feats)[0]   # (N, C', H', W')
-            feats.requires_grad_(False)
-            weights = grad.mean(dim=-2, keepdim=True).mean(dim=-1, keepdim=True)    # (N, C', 1, 1)
-            heatmap = (feats * weights).sum(dim=1, keepdim=True).clamp(0)  # (N, 1, H', W')
-            # heatmap.sub_(heatmap.amin(dim=-2, keepdim=True).amin(dim=-1, keepdim=True))
-            heatmap.div_(heatmap.amax(dim=-2, keepdim=True).amax(dim=-1, keepdim=True))
-            heatmap: torch.Tensor = F.interpolate(heatmap, _input.shape[-2:], mode=mode)[:, 0]   # (N, H, W)
-            # Note that we violate the image order convension (W, H, C)
-        elif method == 'saliency_map':
-            _input.requires_grad_()
-            _output = self(_input).gather(dim=1, index=_label.unsqueeze(1)).sum()
-            grad = torch.autograd.grad(_output, _input)[0]   # (N, C, H, W)
-            _input.requires_grad_(False)
+        match method:
+            case 'grad_cam':
+                feats = self._model.get_fm(_input).detach()   # (N, C', H', W')
+                feats.requires_grad_()
+                _output: torch.Tensor = self._model.pool(feats)   # (N, C', 1, 1)
+                _output = self._model.flatten(_output)   # (N, C')
+                _output = self._model.classifier(_output)   # (N, num_classes)
+                _output = _output.gather(dim=1, index=_label.unsqueeze(1)).sum()
+                grad = torch.autograd.grad(_output, feats)[0]   # (N, C', H', W')
+                feats.requires_grad_(False)
+                weights = grad.mean(dim=-2, keepdim=True).mean(dim=-1, keepdim=True)    # (N, C', 1, 1)
+                heatmap = (feats * weights).sum(dim=1, keepdim=True).clamp(0)  # (N, 1, H', W')
+                # heatmap.sub_(heatmap.amin(dim=-2, keepdim=True).amin(dim=-1, keepdim=True))
+                heatmap.div_(heatmap.amax(dim=-2, keepdim=True).amax(dim=-1, keepdim=True))
+                heatmap: torch.Tensor = F.interpolate(heatmap, _input.shape[-2:], mode=mode)[:, 0]   # (N, H, W)
+                # Note that we violate the image order convension (W, H, C)
+            case 'saliency_map':
+                _input.requires_grad_()
+                _output = self(_input).gather(dim=1, index=_label.unsqueeze(1)).sum()
+                grad = torch.autograd.grad(_output, _input)[0]   # (N, C, H, W)
+                _input.requires_grad_(False)
 
-            heatmap = grad.abs().amax(dim=1)   # (N, H, W)
-            heatmap.sub_(heatmap.amin(dim=-2, keepdim=True).amin(dim=-1, keepdim=True))
-            heatmap.div_(heatmap.amax(dim=-2, keepdim=True).amax(dim=-1, keepdim=True))
+                heatmap = grad.abs().amax(dim=1)   # (N, H, W)
+                heatmap.sub_(heatmap.amin(dim=-2, keepdim=True).amin(dim=-1, keepdim=True))
+                heatmap.div_(heatmap.amax(dim=-2, keepdim=True).amax(dim=-1, keepdim=True))
+            case _:
+                raise NotImplementedError(f'{method=} is not supported yet.')
         heatmap = apply_cmap(heatmap.detach().cpu(), cmap)
         return heatmap[0] if squeeze_flag else heatmap
 
@@ -468,23 +474,23 @@ class ImageModel(Model):
             return adv_x, _label
         return _input, _label
 
-    def _validate(self, adv_train: Union[bool, str] = None, **kwargs) -> tuple[float, float]:
+    def _validate(self, adv_train: None | bool | str = None, **kwargs) -> tuple[float, float]:
         r""""""
         adv_train = bool(adv_train) if adv_train is not None else bool(self.adv_train)
         if not adv_train:
             return super()._validate(**kwargs)
-        _, clean_acc = super()._validate(print_prefix='Validate Clean', main_tag='valid clean',
+        clean_acc, _ = super()._validate(print_prefix='Validate Clean', main_tag='valid clean',
                                          adv_train=False, **kwargs)
-        _, adv_acc = super()._validate(print_prefix='Validate Adv', main_tag='valid adv',
+        adv_acc, _ = super()._validate(print_prefix='Validate Adv', main_tag='valid adv',
                                        adv_train=True, **kwargs)
-        return adv_acc, clean_acc + adv_acc
+        return clean_acc + adv_acc, adv_acc
 
     def _train(self, epochs: int, optimizer: Optimizer, lr_scheduler: _LRScheduler = None,
                adv_train: bool = None,
                lr_warmup_epochs: int = 0,
                model_ema: ExponentialMovingAverage = None,
                model_ema_steps: int = 32,
-               grad_clip: float = None, pre_conditioner: Union[KFAC, EKFAC] = None,
+               grad_clip: float = None, pre_conditioner: None | KFAC | EKFAC = None,
                print_prefix: str = 'Train', start_epoch: int = 0, resume: int = 0,
                validate_interval: int = 10, save: bool = False, amp: bool = False,
                loader_train: torch.utils.data.DataLoader = None,
@@ -499,7 +505,7 @@ class ImageModel(Model):
                folder_path: str = None, suffix: str = None,
                writer=None, main_tag: str = 'train', tag: str = '',
                accuracy_fn: Callable[..., list[float]] = None,
-               verbose: bool = True, indent: int = 0, **kwargs) -> None:
+               verbose: bool = True, indent: int = 0, **kwargs):
         adv_train = adv_train if adv_train is not None else bool(self.adv_train)
         if adv_train:
             after_loss_fn_old = after_loss_fn
@@ -551,48 +557,49 @@ class ImageModel(Model):
                                       amp=amp, scaler=scaler, **kwargs)
             after_loss_fn = after_loss_fn_new
 
-        super()._train(epochs=epochs, optimizer=optimizer, lr_scheduler=lr_scheduler,
-                       adv_train=adv_train,
-                       lr_warmup_epochs=lr_warmup_epochs,
-                       model_ema=model_ema, model_ema_steps=model_ema_steps,
-                       grad_clip=grad_clip, pre_conditioner=pre_conditioner,
-                       print_prefix=print_prefix, start_epoch=start_epoch,
-                       resume=resume, validate_interval=validate_interval,
-                       save=save, amp=amp,
-                       loader_train=loader_train, loader_valid=loader_valid,
-                       epoch_fn=epoch_fn, get_data_fn=get_data_fn,
-                       loss_fn=loss_fn, after_loss_fn=after_loss_fn,
-                       validate_fn=validate_fn,
-                       save_fn=save_fn, file_path=file_path,
-                       folder_path=folder_path, suffix=suffix,
-                       writer=writer, main_tag=main_tag, tag=tag,
-                       accuracy_fn=accuracy_fn,
-                       verbose=verbose, indent=indent, **kwargs)
+        return super()._train(epochs=epochs, optimizer=optimizer, lr_scheduler=lr_scheduler,
+                              adv_train=adv_train,
+                              lr_warmup_epochs=lr_warmup_epochs,
+                              model_ema=model_ema, model_ema_steps=model_ema_steps,
+                              grad_clip=grad_clip, pre_conditioner=pre_conditioner,
+                              print_prefix=print_prefix, start_epoch=start_epoch,
+                              resume=resume, validate_interval=validate_interval,
+                              save=save, amp=amp,
+                              loader_train=loader_train, loader_valid=loader_valid,
+                              epoch_fn=epoch_fn, get_data_fn=get_data_fn,
+                              loss_fn=loss_fn, after_loss_fn=after_loss_fn,
+                              validate_fn=validate_fn,
+                              save_fn=save_fn, file_path=file_path,
+                              folder_path=folder_path, suffix=suffix,
+                              writer=writer, main_tag=main_tag, tag=tag,
+                              accuracy_fn=accuracy_fn,
+                              verbose=verbose, indent=indent, **kwargs)
 
     def adv_loss(self, _input: torch.Tensor, _label: torch.Tensor,
                  loss_fn: Callable[..., torch.Tensor] = None,
                  adv_train: str = None) -> torch.Tensor:
         adv_train = adv_train if adv_train is not None else self.adv_train
         loss_fn = loss_fn if callable(loss_fn) else self.loss
-        if adv_train == 'trades':   # TODO: python 3.10 match
-            noise = 1e-3 * torch.randn_like(_input)
-            org_prob = self.get_prob(_input)
-            adv_x, _ = self.pgd.optimize(_input=_input, noise=noise, target=_label,
-                                         iteration=self.adv_train_iter,
-                                         pgd_alpha=self.adv_train_alpha,
-                                         pgd_eps=self.adv_train_eps,
-                                         loss_fn=self.trades_loss_fn,
-                                         loss_kwargs={'org_prob': org_prob.detach()})
-            adv_x = _input + (adv_x - _input).detach()
-            return loss_fn(_input, _label) - self.adv_train_trades_beta * \
-                self.trades_loss_fn(_input=adv_x, org_prob=org_prob)
-        elif adv_train == 'pgd':
-            adv_x, _ = self.pgd.optimize(_input=_input, target=_label,
-                                         iteration=self.adv_train_iter,
-                                         pgd_alpha=self.adv_train_alpha,
-                                         pgd_eps=self.adv_train_eps,
-                                         random_init=self.adv_train_random_init)
-            adv_x = _input + (adv_x - _input).detach()
-            return loss_fn(adv_x, _label)
-        else:
-            raise NotImplementedError(adv_train)
+        match adv_train:
+            case 'trades':
+                noise = 1e-3 * torch.randn_like(_input)
+                org_prob = self.get_prob(_input)
+                adv_x, _ = self.pgd.optimize(_input=_input, noise=noise, target=_label,
+                                             iteration=self.adv_train_iter,
+                                             pgd_alpha=self.adv_train_alpha,
+                                             pgd_eps=self.adv_train_eps,
+                                             loss_fn=self.trades_loss_fn,
+                                             loss_kwargs={'org_prob': org_prob.detach()})
+                adv_x = _input + (adv_x - _input).detach()
+                return loss_fn(_input, _label) - self.adv_train_trades_beta * \
+                    self.trades_loss_fn(_input=adv_x, org_prob=org_prob)
+            case 'pgd':
+                adv_x, _ = self.pgd.optimize(_input=_input, target=_label,
+                                             iteration=self.adv_train_iter,
+                                             pgd_alpha=self.adv_train_alpha,
+                                             pgd_eps=self.adv_train_eps,
+                                             random_init=self.adv_train_random_init)
+                adv_x = _input + (adv_x - _input).detach()
+                return loss_fn(adv_x, _label)
+            case _:
+                raise NotImplementedError(f'{adv_train=} is not supported yet.')
