@@ -10,8 +10,7 @@ import os
 import tarfile
 import zipfile
 from typing import Any, cast
-from collections.abc import Callable    # TODO: python 3.10
-from typing import Optional, Union  # TODO: python 3.10
+from collections.abc import Callable
 
 
 __all__ = ['MemoryDataset', 'ZipFolder']
@@ -30,7 +29,7 @@ class MemoryDataset(VisionDataset):
             self.targets = list(_dict['targets'])
         assert len(self.data) == len(self.targets)
 
-    def __getitem__(self, index: Union[int, slice]) -> tuple[Any, Any]:
+    def __getitem__(self, index: int | slice) -> tuple[Any, Any]:
         """
         Args:
             index (int): Index
@@ -53,8 +52,8 @@ class MemoryDataset(VisionDataset):
 
 # https://github.com/koenvandesande/vision/blob/read_zipped_data/torchvision/datasets/zippedfolder.py
 class ZipFolder(DatasetFolder):
-    def __init__(self, root: str, transform: Optional[Callable] = None, target_transform: Optional[Callable] = None,
-                 is_valid_file: Optional[Callable[[str], bool]] = None, memory: bool = True) -> None:
+    def __init__(self, root: str, transform: None | Callable = None, target_transform: None | Callable = None,
+                 is_valid_file: None | Callable[[str], bool] = None, memory: bool = True) -> None:
         if not root.lower().endswith('.zip'):
             raise TypeError("Need zip file for data source: ", root)
         if memory:
@@ -85,8 +84,8 @@ class ZipFolder(DatasetFolder):
         self,
         directory: str,
         class_to_idx: dict[str, int],
-        extensions: Optional[tuple[str, ...]] = None,
-        is_valid_file: Optional[Callable[[str], bool]] = None,
+        extensions: None | tuple[str, ...] = None,
+        is_valid_file: None | Callable[[str], bool] = None,
     ) -> list[tuple[str, int]]:
         instances = []
         both_none = extensions is None and is_valid_file is None
@@ -114,29 +113,31 @@ class ZipFolder(DatasetFolder):
         return Image.open(f).convert('RGB')
 
     def _find_classes(self, *args, **kwargs) -> tuple[list[str], dict[str, int]]:
-        """
-        Finds the class folders in a dataset.
+        r"""Finds the class folders in a dataset.
+
         Args:
             dir (str): Root directory path.
+
         Returns:
             tuple: (classes, class_to_idx) where classes are relative to (dir), and class_to_idx is a dictionary.
+
         Ensures:
             No class is a subdirectory of another.
         """
-        classes: set[str] = set()
+        class_set = set()
         for filepath in self.root_data.namelist():
             root, target_class = os.path.split(os.path.dirname(filepath))
             if root:
-                classes.add(target_class)
-        classes = list(classes)
+                class_set.add(target_class)
+        classes = list(class_set)
         classes.sort()  # TODO: Pylance issue
         class_to_idx = {classes[i]: i for i in range(len(classes))}
         return classes, class_to_idx
 
 
 class TarFolder(DatasetFolder):
-    def __init__(self, root: str, transform: Optional[Callable] = None, target_transform: Optional[Callable] = None,
-                 is_valid_file: Optional[Callable[[str], bool]] = None, memory: bool = True) -> None:
+    def __init__(self, root: str, transform: None | Callable = None, target_transform: None | Callable = None,
+                 is_valid_file: None | Callable[[str], bool] = None, memory: bool = True) -> None:
         if not root.lower().endswith('.tar'):
             raise TypeError("Need tar file for data source: ", root)
         if memory:
@@ -167,8 +168,8 @@ class TarFolder(DatasetFolder):
         self,
         directory: str,
         class_to_idx: dict[str, int],
-        extensions: Optional[tuple[str, ...]] = None,
-        is_valid_file: Optional[Callable[[str], bool]] = None,
+        extensions: None | tuple[str, ...] = None,
+        is_valid_file: None | Callable[[str], bool] = None,
     ) -> list[tuple[str, int]]:
         instances: list[tuple[str, int]] = []
         both_none = extensions is None and is_valid_file is None
