@@ -14,7 +14,7 @@ import os
 from abc import ABC, abstractmethod
 
 from typing import TYPE_CHECKING
-from typing import Iterable, Union    # TODO: python 3.10
+from typing import Iterable
 from trojanzoo.configs import Config
 import argparse    # TODO: python 3.10
 from collections.abc import Callable
@@ -125,7 +125,7 @@ class Dataset(ABC, BasicObject):
                  valid_batch_size: int = 100,
                  folder_path: str = None, download: bool = False,
                  split_ratio: float = 0.8, num_workers: int = 4,
-                 loss_weights: Union[bool, np.ndarray, torch.Tensor] = False,
+                 loss_weights: bool | np.ndarray | torch.Tensor = False,
                  **kwargs):
         super().__init__(**kwargs)
         self.param_list['dataset'] = ['num_classes', 'batch_size', 'valid_batch_size',
@@ -157,14 +157,17 @@ class Dataset(ABC, BasicObject):
             raise
         # ----------------------------------------------- #
         # Loss Weights
-        if isinstance(loss_weights, bool):
-            loss_weights = self.get_loss_weights(
-            ) if loss_weights else None    # TODO: issue 5 pylance
-        elif isinstance(loss_weights, np.ndarray):
-            loss_weights = torch.from_numpy(loss_weights).to(device=env['device'])
-        else:
-            loss_weights = loss_weights.to(device=env['device'])
-        self.loss_weights: torch.Tensor = loss_weights
+        match loss_weights:
+            case bool():
+                # TODO: issue 5 pylance
+                loss_weights = self.get_loss_weights() if loss_weights else None
+            case np.ndarray():
+                loss_weights = torch.from_numpy(loss_weights).to(device=env['device'])
+            case torch.Tensor():
+                loss_weights = loss_weights.to(device=env['device'])
+            case _:
+                raise TypeError(type(loss_weights))
+        self.loss_weights: None | torch.Tensor = loss_weights
 
     @functools.cached_property
     def batch_size(self):
@@ -251,7 +254,7 @@ class Dataset(ABC, BasicObject):
         ...
 
     def get_dataset(self, mode: str = None, seed: int = None,
-                    class_list: Union[int, list[int]] = None,
+                    class_list: None | int | list[int] = None,
                     **kwargs):
         r"""Get dataset. Call :meth:`split_dataset` to split the training set
         if :attr:`valid_set` is ``False``.
@@ -285,8 +288,7 @@ class Dataset(ABC, BasicObject):
         return dataset
 
     @staticmethod
-    def split_dataset(dataset: Union[torch.utils.data.Dataset,
-                                     torch.utils.data.Subset],
+    def split_dataset(dataset: torch.utils.data.Dataset | torch.utils.data.Subset,
                       length: int = None, percent: float = None,
                       shuffle: bool = True, seed: int = None):
         r"""Split a dataset into two subsets.
@@ -338,7 +340,7 @@ class Dataset(ABC, BasicObject):
 
     @staticmethod
     def get_class_subset(dataset: torch.utils.data.Dataset,
-                         class_list: Union[int, list[int]]) -> torch.utils.data.Subset:
+                         class_list: int | list[int]) -> torch.utils.data.Subset:
         r"""Get a subset from dataset with certain classes.
 
         Args:
@@ -399,15 +401,15 @@ class Dataset(ABC, BasicObject):
             torch.utils.data.DataLoader: The pytorch dataloader.
         """
         if batch_size is None:
-            # TODO: python 3.10 match
-            if mode == 'train':
-                batch_size = self.batch_size
-            elif mode == 'valid':
-                batch_size = self.valid_batch_size
-            else:
-                raise ValueError(f'{mode=}')
+            match mode:
+                case 'train':
+                    batch_size = self.batch_size
+                case 'valid':
+                    batch_size = self.valid_batch_size
+                case _:
+                    raise ValueError(f'{mode=}')
         if shuffle is None:
-            shuffle = mode == 'train'
+            shuffle = (mode == 'train')
         if num_workers is None:
             num_workers = self.num_workers
         if dataset is None:
@@ -456,7 +458,7 @@ class Dataset(ABC, BasicObject):
 
 
 def add_argument(parser: argparse.ArgumentParser, dataset_name: str = None,
-                 dataset: Union[str, Dataset] = None,
+                 dataset: str | Dataset = None,
                  config: Config = config,
                  class_dict: dict[str, type[Dataset]] = {}
                  ) -> argparse._ArgumentGroup:
@@ -467,7 +469,7 @@ def add_argument(parser: argparse.ArgumentParser, dataset_name: str = None,
     Args:
         parser (argparse.ArgumentParser): The parser to add arguments.
         dataset_name (str): The dataset name.
-        dataset (str | Dataset): Dataset Instance or dataset name
+        dataset (str | Dataset): Dataset instance or dataset name
             (as the alias of `dataset_name`).
         config (Config): The default parameter config,
             which contains the default dataset name if not provided.
@@ -512,7 +514,7 @@ def create(dataset_name: str = None, dataset: str = None,
             passed to dataset init method.
 
     Returns:
-        Dataset: Dataset Instance.
+        Dataset: Dataset instance.
     """
     dataset_name = get_name(
         name=dataset_name, module=dataset, arg_list=['-d', '--dataset'])
