@@ -21,7 +21,7 @@ from collections.abc import Iterable
 
 from typing import TYPE_CHECKING
 # TODO: python 3.10
-from typing import Generator, Iterator, Mapping, Optional, Set, Union
+from typing import Generator, Iterator, Mapping
 from trojanzoo.configs import Config    # TODO: python 3.10
 from trojanzoo.utils.model import ExponentialMovingAverage
 from torch.optim.optimizer import Optimizer
@@ -277,7 +277,7 @@ class Model(BasicObject):
         return group
 
     def __init__(self, name: str = 'model', suffix: str = None,
-                 model: Union[type[_Model], _Model] = _Model,
+                 model: type[_Model] | _Model = _Model,
                  dataset: Dataset = None,
                  num_classes: int = None, folder_path: str = None,
                  official: bool = False, pretrained: bool = False,
@@ -310,18 +310,17 @@ class Model(BasicObject):
         if dataset:
             if not isinstance(dataset, Dataset):
                 raise TypeError(f'{type(dataset)=}    {dataset=}')
-            num_classes = num_classes if num_classes is not None \
-                else dataset.num_classes
-            loss_weights = loss_weights if 'loss_weights' in kwargs.keys() \
-                else dataset.loss_weights
+            num_classes = num_classes or dataset.num_classes
+            loss_weights = kwargs.get('loss_weights', dataset.loss_weights)
         self.num_classes = num_classes  # number of classes
         # TODO: what device shall we save loss_weights?
         # numpy, torch, or torch.cuda.
 
-        if isinstance(loss_weights, np.ndarray):
-            loss_weights = torch.from_numpy(loss_weights)
-        if isinstance(loss_weights, torch.Tensor):
-            loss_weights = loss_weights.to(device=env['device'])
+        match loss_weights:
+            case np.ndarray():
+                loss_weights = torch.from_numpy(loss_weights).to(device=env['device'])
+            case torch.Tensor():
+                loss_weights = loss_weights.to(device=env['device'])
 
         self.loss_weights = loss_weights
         self.layer_name_list: list[str] = None
@@ -331,13 +330,15 @@ class Model(BasicObject):
         self.criterion_noreduction = self.define_criterion(
             weight=loss_weights, reduction='none')
         self.softmax = nn.Softmax(dim=1)
-        if isinstance(model, type):
-            if num_classes is not None:
-                kwargs['num_classes'] = num_classes
-            self._model = model(name=name, dataset=dataset, **kwargs)
-        else:
-            assert isinstance(model, nn.Module)
-            self._model = model
+        match model:
+            case type():
+                if num_classes is not None:
+                    kwargs['num_classes'] = num_classes
+                self._model = model(name=name, dataset=dataset, **kwargs)
+            case nn.Module():
+                self._model = model
+            case _:
+                raise TypeError(type(model))
         self.model = self.get_parallel_model(self._model)
         self.activate_params([])
         if official:
@@ -350,18 +351,22 @@ class Model(BasicObject):
 
     # ----------------- Forward Operations ----------------------#
 
-    def get_logits(self, _input: torch.Tensor, randomized_smooth: bool = None,
+    def get_logits(self, _input: torch.Tensor, parallel: bool = False,
+                   randomized_smooth: bool = None,
                    rs_sigma: float = None, rs_n: int = None,
                    **kwargs) -> torch.Tensor:
         r"""Get logits of :attr:`_input`.
 
         Note:
-            Users should used model as Callable function
-            rather than calling this method directly,
+            Users should use model as Callable function
+            rather than call this method directly,
             because ``__call__`` supports :any:`torch.cuda.amp`.
 
         Args:
             _input (torch.Tensor): The batched input tensor.
+            parallel (bool): Whether to use parallel model
+                `self.model` rather than `self._model`.
+                Defautls to ``False``.
             randomized_smooth (bool | None): Whether to use randomized smoothing.
                 If it's ``None``, use :attr:`self.randmized_smooth` instead.
                 Defaults to ``None``.
@@ -376,6 +381,7 @@ class Model(BasicObject):
         Returns:
             torch.Tensor: The logit tensor with shape ``(N, C)``.
         """
+        model = self.model if parallel else self._model
         if randomized_smooth is None:
             randomized_smooth = self.randomized_smooth
         if randomized_smooth:
@@ -385,15 +391,15 @@ class Model(BasicObject):
             for _ in range(rs_n):
                 # TODO: valid input clip issue
                 _input_noise = add_noise(_input, std=rs_sigma)
-                _list.append(self.model(_input_noise, **kwargs))
+                _list.append(model(_input_noise, **kwargs))
             return torch.stack(_list).mean(dim=0)
             # TODO: memory issues and parallel possibilities
             # _input_noise = add_noise(repeat_to_batch(
             #     _input, batch_size=n), std=sigma).flatten(end_dim=1)
-            # return self.model(_input_noise, **kwargs).view(
+            # return model(_input_noise, **kwargs).view(
             #     n, len(_input), self.num_classes).mean(dim=0)
         else:
-            return self.model(_input, **kwargs)
+            return model(_input, **kwargs)
 
     def get_fm(self, _input: torch.Tensor, **kwargs) -> torch.Tensor:
         r"""Get the feature map of :attr:`_input`,
@@ -441,7 +447,7 @@ class Model(BasicObject):
         return self.softmax(self(_input, **kwargs))
 
     def get_target_prob(self, _input: torch.Tensor,
-                        target: Union[torch.Tensor, list[int], int],
+                        target: int | list[int] | torch.Tensor,
                         **kwargs) -> torch.Tensor:
         r"""Get the probability w.r.t. :attr:`target` class of :attr:`_input`
         (using :any:`torch.gather`).
@@ -449,16 +455,17 @@ class Model(BasicObject):
         Args:
             _input (torch.Tensor): The batched input tensor
                 passed to :meth:`_Model.get_logits()`.
-            target (torch.Tensor | list[int] | int): Batched target classes.
+            target (int | list[int] | torch.Tensor): Batched target classes.
             **kwargs: Keyword arguments passed to :meth:`get_logits()`.
 
         Returns:
             torch.Tensor: The probability tensor with shape ``(N)``.
         """
-        if isinstance(target, int):
-            target = [target] * len(_input)
-        if isinstance(target, list):
-            target = torch.tensor(target, device=_input.device)
+        match target:
+            case int():
+                target = [target] * len(_input)
+            case list():
+                target = torch.tensor(target, device=_input.device)
         return self.get_prob(_input, **kwargs).gather(
             dim=1, index=target.unsqueeze(1)).flatten()
 
@@ -580,12 +587,13 @@ class Model(BasicObject):
             :func:`trojanzoo.utils.model.get_layer()`.
         """
         if layer_input == 'input':
-            if layer_output == 'classifier':
-                return self(_input)
-            elif layer_output == 'features':
-                return self._model.get_fm(_input)
-            elif layer_output == 'flatten':
-                return self.get_final_fm(_input)
+            match layer_output:
+                case 'classifier':
+                    return self(_input)
+                case 'features':
+                    return self._model.get_fm(_input)
+                case 'flatten':
+                    return self.get_final_fm(_input)
         if self.layer_name_list is None:
             self.layer_name_list: list[str] = self.get_layer_name(
                 use_filter=False, non_leaf=True)
@@ -609,6 +617,9 @@ class Model(BasicObject):
             _output (torch.Tensor | None): The logits of :attr:`_input`.
                 If ``None``, use :attr:`_input` to calculate logits.
                 Defaults to ``None``.
+            reduction (str): Specifies the reduction to apply to the output.
+                Choose from ``['none', 'mean']``.
+                Defaults to ``'mean'``.
             **kwargs: Keyword arguments passed to :meth:`get_logits()`
                 if :attr:`_output` is not provided.
 
@@ -625,8 +636,8 @@ class Model(BasicObject):
     # -------------------------------------------------------- #
 
     def define_optimizer(
-            self, parameters: Union[str, Iterator[nn.Parameter]] = 'full',
-            OptimType: Union[str, type[Optimizer]] = 'SGD',
+            self, parameters: str | Iterator[nn.Parameter] = 'full',
+            OptimType: str | type[Optimizer] = 'SGD',
             lr: float = 0.1, momentum: float = 0.0, weight_decay: float = 0.0,
             lr_scheduler: bool = False,
             lr_scheduler_type: str = 'CosineAnnealingLR',
@@ -693,10 +704,13 @@ class Model(BasicObject):
         """
         kwargs['momentum'] = momentum
         kwargs['weight_decay'] = weight_decay
-        if isinstance(parameters, str):
-            parameters = self.get_parameter_from_name(name=parameters)
-        if not isinstance(parameters, Iterable):
-            raise TypeError(f'{type(parameters)=}    {parameters=}')
+        match parameters:
+            case str():
+                parameters = self.get_parameter_from_name(name=parameters)
+            case Iterable():
+                pass
+            case _:
+                raise TypeError(f'{type(parameters)=}    {parameters=}')
         if isinstance(OptimType, str):
             OptimType: type[Optimizer] = getattr(torch.optim, OptimType)
         keys = OptimType.__init__.__code__.co_varnames
@@ -704,34 +718,36 @@ class Model(BasicObject):
         optimizer = OptimType(parameters, lr, **kwargs)
         _lr_scheduler: _LRScheduler = None
         if lr_scheduler:
-            main_lr_scheduler: _LRScheduler = None
-            if lr_scheduler_type == 'StepLR':    # TODO: python 3.10 match
-                main_lr_scheduler = torch.optim.lr_scheduler.StepLR(
-                    optimizer, step_size=lr_step_size, gamma=lr_gamma)
-            elif lr_scheduler_type == 'CosineAnnealingLR':
-                main_lr_scheduler = torch.optim.lr_scheduler.CosineAnnealingLR(
-                    optimizer, T_max=epochs - lr_warmup_epochs, eta_min=lr_min)
-            elif lr_scheduler_type == 'ExponentialLR':
-                main_lr_scheduler = torch.optim.lr_scheduler.ExponentialLR(
-                    optimizer, gamma=lr_gamma)
-            else:
-                raise RuntimeError(
-                    f'Invalid lr scheduler "{lr_scheduler_type}".'
-                    'Only StepLR, CosineAnnealingLR and ExponentialLR '
-                    'are supported.')
+            main_lr_scheduler: _LRScheduler
+            match lr_scheduler_type:
+                case 'StepLR':
+                    main_lr_scheduler = torch.optim.lr_scheduler.StepLR(
+                        optimizer, step_size=lr_step_size, gamma=lr_gamma)
+                case 'CosineAnnealingLR':
+                    main_lr_scheduler = torch.optim.lr_scheduler.CosineAnnealingLR(
+                        optimizer, T_max=epochs - lr_warmup_epochs, eta_min=lr_min)
+                case 'ExponentialLR':
+                    main_lr_scheduler = torch.optim.lr_scheduler.ExponentialLR(
+                        optimizer, gamma=lr_gamma)
+                case _:
+                    raise NotImplementedError(
+                        f'Invalid {lr_scheduler_type=}.'
+                        'Only "StepLR", "CosineAnnealingLR" and "ExponentialLR" '
+                        'are supported.')
             if lr_warmup_epochs > 0:
-                if lr_warmup_method == "linear":
-                    warmup_lr_scheduler = torch.optim.lr_scheduler.LinearLR(
-                        optimizer, start_factor=lr_warmup_decay,
-                        total_iters=lr_warmup_epochs)
-                elif lr_warmup_method == "constant":
-                    warmup_lr_scheduler = torch.optim.lr_scheduler.ConstantLR(
-                        optimizer, factor=lr_warmup_decay,
-                        total_iters=lr_warmup_epochs)
-                else:
-                    raise RuntimeError(
-                        f'Invalid warmup lr method "{lr_warmup_method}".'
-                        'Only linear and constant are supported.')
+                match lr_warmup_method:
+                    case 'linear':
+                        warmup_lr_scheduler = torch.optim.lr_scheduler.LinearLR(
+                            optimizer, start_factor=lr_warmup_decay,
+                            total_iters=lr_warmup_epochs)
+                    case 'constant':
+                        warmup_lr_scheduler = torch.optim.lr_scheduler.ConstantLR(
+                            optimizer, factor=lr_warmup_decay,
+                            total_iters=lr_warmup_epochs)
+                    case _:
+                        raise NotImplementedError(
+                            f'Invalid {lr_warmup_method=}.'
+                            'Only "linear" and "constant" are supported.')
                 _lr_scheduler = torch.optim.lr_scheduler.SequentialLR(
                     optimizer,
                     schedulers=[warmup_lr_scheduler, main_lr_scheduler],
@@ -771,7 +787,7 @@ class Model(BasicObject):
     @torch.no_grad()
     def load(self, file_path: str = None, folder_path: str = None,
              suffix: str = None, inplace: bool = True,
-             map_location: Union[str, Callable, torch.device, dict] = 'cpu',
+             map_location: str | Callable | torch.device | dict = 'cpu',
              component: str = 'full', strict: bool = True,
              verbose: bool = False, indent: int = 0,
              **kwargs) -> OrderedDict[str, torch.Tensor]:
@@ -837,18 +853,19 @@ class Model(BasicObject):
                 print(f'{file_path=}')
                 raise
         module = self._model
-        if component == 'features':  # TODO: python 3.10 match
-            module = self._model.features
-            _dict = OrderedDict(
-                [(key.removeprefix('features.'), value)
-                    for key, value in _dict.items()])
-        elif component == 'classifier':
-            module = self._model.classifier
-            _dict = OrderedDict(
-                [(key.removeprefix('classifier.'), value)
-                    for key, value in _dict.items()])
-        else:
-            assert component == 'full', f'{component=}'
+        match component:
+            case 'features':
+                module = self._model.features
+                _dict = OrderedDict(
+                    [(key.removeprefix('features.'), value)
+                        for key, value in _dict.items()])
+            case 'classifier':
+                module = self._model.classifier
+                _dict = OrderedDict(
+                    [(key.removeprefix('classifier.'), value)
+                        for key, value in _dict.items()])
+            case _:
+                assert component == 'full', f'{component=}'
         if inplace:
             try:
                 module.load_state_dict(_dict, strict=strict)
@@ -911,8 +928,7 @@ class Model(BasicObject):
                 f'Model {self.name} saved at: {file_path}', indent=indent)
 
     def get_official_weights(self, url: str = None,
-                             map_location: Union[str, Callable,
-                                                 torch.device, dict] = 'cpu',
+                             map_location: str | Callable | torch.device | dict = 'cpu',
                              **kwargs) -> OrderedDict[str, torch.Tensor]:
         r"""Get official model weights from :attr:`url`.
 
@@ -939,7 +955,7 @@ class Model(BasicObject):
                lr_warmup_epochs: int = 0,
                model_ema: ExponentialMovingAverage = None,
                model_ema_steps: int = 32,
-               grad_clip: float = None, pre_conditioner: Union[KFAC, EKFAC] = None,
+               grad_clip: float = None, pre_conditioner: None | KFAC | EKFAC = None,
                print_prefix: str = 'Train', start_epoch: int = 0, resume: int = 0,
                validate_interval: int = 10, save: bool = False, amp: bool = False,
                loader_train: torch.utils.data.DataLoader = None,
@@ -954,7 +970,7 @@ class Model(BasicObject):
                folder_path: str = None, suffix: str = None,
                writer=None, main_tag: str = 'train', tag: str = '',
                accuracy_fn: Callable[..., list[float]] = None,
-               verbose: bool = True, indent: int = 0, **kwargs) -> None:
+               verbose: bool = True, indent: int = 0, **kwargs):
         r"""Train the model"""
         loader_train = loader_train if loader_train is not None \
             else self.dataset.loader['train']
@@ -1002,7 +1018,7 @@ class Model(BasicObject):
         r"""Evaluate the model.
 
         Returns:
-            (float, float): Loss and accuracy.
+            (float, float): Accuracy and loss.
         """
         module = self._model if module is None else module
         num_classes = self.num_classes if num_classes is None else num_classes
@@ -1090,7 +1106,7 @@ class Model(BasicObject):
     # since they are calling their own nn.DataParallel.
     # TODO: nn.parallel.DistributedDataParallel
     @staticmethod
-    def get_parallel_model(_model: _Model) -> Union[_Model, nn.DataParallel]:
+    def get_parallel_model(_model: _Model) -> _Model | nn.DataParallel:
         r"""Get the parallel model if there are more than 1 GPU avaiable.
 
         Warning:
@@ -1167,7 +1183,7 @@ class Model(BasicObject):
         self.model.cpu()
         return self
 
-    def cuda(self, device: Optional[Union[int, torch.device]] = None):
+    def cuda(self, device: None | int | torch.device = None):
         r"""Moves all model parameters and buffers to the GPU.
 
         See Also:
@@ -1247,7 +1263,7 @@ class Model(BasicObject):
         """
         return self._model.modules()
 
-    def named_modules(self, memo: Optional[Set[nn.Module]] = None,
+    def named_modules(self, memo: None | set[nn.Module] = None,
                       prefix: str = ''
                       ) -> Generator[tuple[str, nn.Module], None, None]:
         r"""Returns an iterator over all modules in the network, yielding
@@ -1325,13 +1341,15 @@ class Model(BasicObject):
 
     def get_parameter_from_name(self, name: str = 'full'
                                 ) -> Iterator[nn.Parameter]:
-        params = self._model.parameters()
-        if name == 'features':  # TODO: python 3.10 match
-            params = self._model.features.parameters()
-        elif name in ['classifier', 'partial']:
-            params = self._model.classifier.parameters()
-        elif name != 'full':
-            raise NotImplementedError(f'{name=}')
+        match name:
+            case 'features':
+                params = self._model.features.parameters()
+            case 'classifier' | 'partial':
+                params = self._model.classifier.parameters()
+            case 'full':
+                params = self._model.parameters()
+            case _:
+                raise NotImplementedError(f'{name=}')
         return params
 
     def __call__(self, _input: torch.Tensor, amp: bool = False,
@@ -1342,8 +1360,9 @@ class Model(BasicObject):
         return self.get_logits(_input, **kwargs)
 
 
-def add_argument(parser: argparse.ArgumentParser, model_name: str = None,
-                 model: Union[str, Model] = None,
+def add_argument(parser: argparse.ArgumentParser,
+                 model_name: None | str = None,
+                 model: None | str | Model = None,
                  config: Config = config,
                  class_dict: dict[str, type[Model]] = {}
                  ) -> argparse._ArgumentGroup:
@@ -1386,8 +1405,8 @@ def add_argument(parser: argparse.ArgumentParser, model_name: str = None,
     return ModelType.add_argument(group)
 
 
-def create(model_name: str = None, model: Union[str, Model] = None,
-           dataset_name: str = None, dataset: Union[str, Dataset] = None,
+def create(model_name: None | str = None, model: None | str | Model = None,
+           dataset_name: None | str = None, dataset: None | str | Dataset = None,
            config: Config = config,
            class_dict: dict[str, type[Model]] = {},
            **kwargs) -> Model:
@@ -1405,7 +1424,7 @@ def create(model_name: str = None, model: Union[str, Model] = None,
             (as the alias of `model_name`).
         dataset_name (str): The dataset name.
         dataset (str | trojanzoo.datasets.Dataset):
-            Dataset Instance or dataset name
+            Dataset instance or dataset name
             (as the alias of `dataset_name`).
         config (Config): The default parameter config.
         class_dict (dict[str, type[Model]]):
@@ -1447,7 +1466,7 @@ def create(model_name: str = None, model: Union[str, Model] = None,
         result['folder_path'] = os.path.join(result['model_dir'],
                                              dataset.data_type,
                                              dataset.name)
-    return ModelType(name=model_name, dataset=dataset, **result)
+    return ModelType(name=model_name, dataset=dataset, dataset_name=dataset_name, **result)
 
 
 def output_available_models(class_dict: dict[str, type[Model]] = {},
