@@ -162,9 +162,16 @@ def get_all_layer(module: nn.Module, x: torch.Tensor,
             of :any:`torch.nn.Sequential`.
             If ``False``, will traverse children of all :any:`torch.nn.Module`.
             Defaults to ``False``.
-        verbose (bool): Whether to show verbose information
-            including layer names and output shape.
-            Defaults to ``True``.
+        verbose (int): The output level to show information
+            including layer name, output shape and module information.
+            Setting it larger than ``0`` will enable the output.
+            Different integer values stands for different module information.
+            Defaults to ``0``.
+
+            * ``0``: No output
+            * ``1``: Show layer class name.
+            * ``2``: Show layer string (first line).
+            * ``3``: Show layer string (full).
 
     Returns:
         dict[str, torch.Tensor]: The dict of all layer outputs.
@@ -250,14 +257,15 @@ def _get_all_layer(module: nn.Module, x: torch.Tensor,
         if verbose:
             shape_str = str(list(x.shape))
             module_str = ''
-            if verbose == 1:
-                module_str = module.__class__.__name__
-            elif verbose == 2:
-                module_str = type(module)
-            elif verbose == 3:
-                module_str = str(module).split('\n')[0].removesuffix('(')
-            else:
-                module_str = str(module)
+            match verbose:
+                case 1:
+                    module_str = module.__class__.__name__
+                case 2:
+                    module_str = type(module)
+                case 3:
+                    module_str = str(module).split('\n')[0].removesuffix('(')
+                case _:
+                    module_str = str(module)
             print(f'{ansi["blue_light"]}{prefix:<50s}{ansi["reset"]}'
                   f'{ansi["yellow"]}{shape_str:<20}{ansi["reset"]}'
                   f'{module_str}')
