@@ -5,10 +5,9 @@ from .process import BasicObject, Process, ModelProcess
 from trojanzoo.utils.output import ansi, prints
 
 import sys
-from typing import Union    # TODO: python 3.10
 
 
-def get_name(name: str = None, module: Union[str, object] = None,
+def get_name(name: str = None, module: str | object = None,
              arg_list: list[str] = []) -> str:
     if module is not None:
         if isinstance(module, str):
